@@ -7,7 +7,7 @@
 import logging
 import re
 import sys
-from abc import ABC, abstractstaticmethod
+from abc import ABC, abstractmethod
 from datetime import datetime
 from functools import wraps
 from io import BytesIO
@@ -26,17 +26,18 @@ from typing import (
 from warnings import warn
 
 from lxml import etree
-from lxml.etree import QName, XMLParser, XMLSyntaxError, parse
+from lxml.etree import QName, XMLSyntaxError
 
 from .. import Name, Stream, String
 from .. import __version__ as pikepdf_version
+from .._xml import parse_xml
 
-if sys.version_info < (3, 9):
+if sys.version_info < (3, 9):  # pragma: no cover
     from typing import Iterable, MutableMapping
 else:
     from collections.abc import Iterable, MutableMapping
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from pikepdf import Pdf
 
 XMP_NS_DC = "http://purl.org/dc/elements/1.1/"
@@ -93,7 +94,7 @@ log = logging.getLogger(__name__)
 
 
 class NeverRaise(Exception):
-    """An exception that is never raised"""
+    """An exception that is never raised."""
 
     pass  # pylint: disable=unnecessary-pass
 
@@ -128,6 +129,23 @@ re_xml_illegal_bytes = re.compile(
 )
 
 
+def _parser_basic(xml: bytes):
+    return parse_xml(BytesIO(xml))
+
+
+def _parser_strip_illegal_bytes(xml: bytes):
+    return parse_xml(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
+
+
+def _parser_recovery(xml: bytes):
+    return parse_xml(BytesIO(xml), recover=True)
+
+
+def _parser_replace_with_empty_xmp(_xml: bytes = b''):
+    log.warning("Error occurred parsing XMP, replacing with empty XMP.")
+    return _parser_basic(XMP_EMPTY)
+
+
 def _clean(s: Union[str, Iterable[str]], joiner: str = '; ') -> str:
     """Ensure an object can safely be inserted in a XML tag body.
 
@@ -135,12 +153,15 @@ def _clean(s: Union[str, Iterable[str]], joiner: str = '; ') -> str:
     join it, because it's apparently calling for a new node in a place that
     isn't allowed in the spec or not supported.
     """
-    if not isinstance(s, str) and isinstance(s, Iterable):
-        warn(f"Merging elements of {s}")
-        if isinstance(s, Set):
-            s = joiner.join(sorted(s))
+    if not isinstance(s, str):
+        if isinstance(s, Iterable):
+            warn(f"Merging elements of {s}")
+            if isinstance(s, Set):
+                s = joiner.join(sorted(s))
+            else:
+                s = joiner.join(s)
         else:
-            s = joiner.join(s)
+            raise TypeError("object must be a string or iterable of strings")
     return re_xml_illegal_chars.sub('', s)
 
 
@@ -202,18 +223,20 @@ def decode_pdf_date(s: str) -> datetime:
 
 
 class Converter(ABC):
-    @abstractstaticmethod
-    def xmp_from_docinfo(docinfo_val: Optional[str]) -> Any:
+    @staticmethod
+    @abstractmethod
+    def xmp_from_docinfo(docinfo_val: Optional[str]) -> Any:  # type: ignore
         "Derive XMP metadata from a DocumentInfo string"
 
-    @abstractstaticmethod
+    @staticmethod
+    @abstractmethod
     def docinfo_from_xmp(xmp_val: Any) -> Optional[str]:
         "Derive a DocumentInfo value from equivalent XMP metadata"
 
 
 class AuthorConverter(Converter):
     @staticmethod
-    def xmp_from_docinfo(docinfo_val):
+    def xmp_from_docinfo(docinfo_val: Optional[str]) -> Any:  # type: ignore
         return [docinfo_val]
 
     @staticmethod
@@ -226,14 +249,14 @@ class AuthorConverter(Converter):
             return '; '.join(xmp_val)
 
 
-def _fromisoformat_py36(datestr: str) -> datetime:
+def _fromisoformat_py36(date_string: str) -> datetime:
     """Backported equivalent of datetime.fromisoformat
 
     Can remove whenever we drop Python 3.6 support.
     """
     # strptime %z can't parse a timezone with punctuation
-    if re.search(r'[+-]\d{2}[-:]\d{2}$', datestr):
-        datestr = datestr[:-3] + datestr[-2:]
+    if re.search(r'[+-]\d{2}[-:]\d{2}$', date_string):
+        date_string = date_string[:-3] + date_string[-2:]
     formats = [
         "%Y-%m-%dT%H:%M:%S%z",
         "%Y-%m-%dT%H:%M:%S",
@@ -242,16 +265,16 @@ def _fromisoformat_py36(datestr: str) -> datetime:
     ]
     for fmt in formats:
         try:
-            return datetime.strptime(datestr, fmt)
+            return datetime.strptime(date_string, fmt)
         except ValueError:
             continue
     raise ValueError(
-        f"Could not parse ISO date: {datestr}. Try Python 3.7+, which "
+        f"Could not parse ISO date: {date_string}. Try Python 3.7+, which "
         "has better support for ISO dates."
     )
 
 
-if sys.version_info < (3, 7):
+if sys.version_info < (3, 7) or sys.implementation.name == 'pypy':
     fromisoformat = _fromisoformat_py36
 else:
     fromisoformat = datetime.fromisoformat
@@ -334,6 +357,14 @@ class PdfMetadata(MutableMapping):
     NS: Dict[str, str] = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
     REVERSE_NS: Dict[str, str] = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
 
+    _PARSERS_OVERWRITE_INVALID_XML: Iterable[Callable[[bytes], Any]] = [
+        _parser_basic,
+        _parser_strip_illegal_bytes,
+        _parser_recovery,
+        _parser_replace_with_empty_xmp,
+    ]
+    _PARSERS_STANDARD: Iterable[Callable[[bytes], Any]] = [_parser_basic]
+
     def __init__(
         self,
         pdf: 'Pdf',
@@ -364,6 +395,13 @@ class PdfMetadata(MutableMapping):
         approximately equivalent to certain XMP records. This method copies
         those entries into the XMP metadata.
         """
+
+        def warn_or_raise(msg, e=None):
+            if raise_failure:
+                raise ValueError(msg) from e
+            else:
+                warn(msg)
+
         for uri, shortkey, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, shortkey)
             # docinfo might be a dict or pikepdf.Dictionary, so lookup keys
@@ -380,60 +418,36 @@ class PdfMetadata(MutableMapping):
                 if not val:
                     continue
                 self._setitem(qname, val, True)
-            except (ValueError, AttributeError) as e:
-                msg = f"The metadata field {docinfo_name} could not be copied to XMP"
-                if raise_failure:
-                    raise ValueError(msg) from e
-                else:
-                    warn(msg)
-        valid_docinfo_names = set(
+            except (ValueError, AttributeError, NotImplementedError) as e:
+                warn_or_raise(
+                    f"The metadata field {docinfo_name} could not be copied to XMP", e
+                )
+        valid_docinfo_names = {
             str(docinfo_name) for _, _, docinfo_name, _ in self.DOCINFO_MAPPING
-        )
-        extra_docinfo_names = set(str(k) for k in docinfo.keys()) - valid_docinfo_names
+        }
+        extra_docinfo_names = {str(k) for k in docinfo.keys()} - valid_docinfo_names
         for extra in extra_docinfo_names:
-            msg = (
+            warn_or_raise(
                 f"The metadata field {extra} with value '{repr(docinfo.get(extra))}' "
-                "has no XMP equivalent, so it was discarded"
+                "has no XMP equivalent, so it was discarded",
             )
-            if raise_failure:
-                raise ValueError(msg)
-            else:
-                warn(msg)
 
     def _load(self) -> None:
         try:
             data = self._pdf.Root.Metadata.read_bytes()
         except AttributeError:
-            data = XMP_EMPTY
+            data = b''
         self._load_from(data)
 
     def _load_from(self, data: bytes) -> None:
         if data.strip() == b'':
             data = XMP_EMPTY  # on some platforms lxml chokes on empty documents
 
-        def basic_parser(xml):
-            return parse(BytesIO(xml))
-
-        def strip_illegal_bytes_parser(xml):
-            return parse(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
-
-        def recovery_parser(xml):
-            parser = XMLParser(recover=True)
-            return parse(BytesIO(xml), parser)
-
-        def replace_with_empty_xmp(_xml=None):
-            log.warning("Error occurred parsing XMP, replacing with empty XMP.")
-            return basic_parser(XMP_EMPTY)
-
-        if self.overwrite_invalid_xml:
-            parsers: Iterable[Callable] = [
-                basic_parser,
-                strip_illegal_bytes_parser,
-                recovery_parser,
-                replace_with_empty_xmp,
-            ]
-        else:
-            parsers = [basic_parser]
+        parsers = (
+            self._PARSERS_OVERWRITE_INVALID_XML
+            if self.overwrite_invalid_xml
+            else self._PARSERS_STANDARD
+        )
 
         for parser in parsers:
             try:
@@ -442,19 +456,23 @@ class PdfMetadata(MutableMapping):
                 if str(e).startswith("Start tag expected, '<' not found") or str(
                     e
                 ).startswith("Document is empty"):
-                    self._xmp = replace_with_empty_xmp()
+                    self._xmp = _parser_replace_with_empty_xmp()
                     break
             else:
                 break
 
-        try:
-            pis = self._xmp.xpath('/processing-instruction()')
-            for pi in pis:
-                etree.strip_tags(self._xmp, pi.tag)
-            self._get_rdf_root()
-        except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
-            log.warning("Error occurred parsing XMP", exc_info=e)
-            self._xmp = replace_with_empty_xmp()
+        if self._xmp is not None:
+            try:
+                pis = self._xmp.xpath('/processing-instruction()')
+                for pi in pis:
+                    etree.strip_tags(self._xmp, pi.tag)
+                self._get_rdf_root()
+            except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
+                log.warning("Error occurred parsing XMP", exc_info=e)
+                self._xmp = _parser_replace_with_empty_xmp()
+        else:
+            log.warning("Error occurred parsing XMP")
+            self._xmp = _parser_replace_with_empty_xmp()
         return
 
     @ensure_loaded
@@ -547,7 +565,8 @@ class PdfMetadata(MutableMapping):
         if self.sync_docinfo:
             self._update_docinfo()
 
-    def _qname(self, name: Union[QName, str]) -> str:
+    @classmethod
+    def _qname(cls, name: Union[QName, str]) -> str:
         """Convert name to an XML QName
 
         e.g. pdf:Producer -> {http://ns.adobe.com/pdf/1.3/}Producer
@@ -561,7 +580,7 @@ class PdfMetadata(MutableMapping):
         if name.startswith('{'):
             return name
         prefix, tag = name.split(':', maxsplit=1)
-        uri = self.NS[prefix]
+        uri = cls.NS[prefix]
         return str(QName(uri, tag))
 
     def _prefix_from_uri(self, uriname):
@@ -650,10 +669,7 @@ class PdfMetadata(MutableMapping):
 
     @ensure_loaded
     def __contains__(self, key: Union[str, QName]):
-        try:
-            return any(self._get_element_values(key))
-        except KeyError:
-            return False
+        return any(self._get_element_values(key))
 
     @ensure_loaded
     def __getitem__(self, key: Union[str, QName]):
@@ -684,6 +700,16 @@ class PdfMetadata(MutableMapping):
             raise RuntimeError("Metadata not opened for editing, use with block")
 
         qkey = self._qname(key)
+        self._setitem_check_args(key, val, applying_mark, qkey)
+
+        try:
+            # Update existing node
+            self._setitem_update(key, val, qkey)
+        except StopIteration:
+            # Insert a new node
+            self._setitem_insert(key, val)
+
+    def _setitem_check_args(self, key, val, applying_mark: bool, qkey: str) -> None:
         if (
             self.mark
             and not applying_mark
@@ -703,64 +729,69 @@ class PdfMetadata(MutableMapping):
         if isinstance(val, str) and qkey in (self._qname('dc:creator')):
             log.error(f"{key} should be set to a list of strings")
 
-        def add_array(node, items: Iterable):
-            rdf_type = next(
-                c.rdf_type for c in XMP_CONTAINERS if isinstance(items, c.py_type)
-            )
-            seq = etree.SubElement(node, str(QName(XMP_NS_RDF, rdf_type)))
-            tag_attrib: Optional[Dict[str, str]] = None
-            if rdf_type == 'Alt':
-                tag_attrib = {str(QName(XMP_NS_XML, 'lang')): 'x-default'}
-            for item in items:
-                el = etree.SubElement(
-                    seq, str(QName(XMP_NS_RDF, 'li')), attrib=tag_attrib
-                )
-                el.text = _clean(item)
+    def _setitem_add_array(self, node, items: Iterable) -> None:
+        rdf_type = next(
+            c.rdf_type for c in XMP_CONTAINERS if isinstance(items, c.py_type)
+        )
+        seq = etree.SubElement(node, str(QName(XMP_NS_RDF, rdf_type)))
+        tag_attrib: Optional[Dict[str, str]] = None
+        if rdf_type == 'Alt':
+            tag_attrib = {str(QName(XMP_NS_XML, 'lang')): 'x-default'}
+        for item in items:
+            el = etree.SubElement(seq, str(QName(XMP_NS_RDF, 'li')), attrib=tag_attrib)
+            el.text = _clean(item)
 
-        try:
-            # Locate existing node to replace
-            node, attrib, _oldval, _parent = next(self._get_elements(key))
-            if attrib:
-                if not isinstance(val, str):
-                    raise TypeError(val)
-                node.set(attrib, _clean(val))
-            elif isinstance(val, (list, set)):
-                for child in node.findall('*'):
-                    node.remove(child)
-                add_array(node, val)
-            elif isinstance(val, str):
-                for child in node.findall('*'):
-                    node.remove(child)
-                if str(self._qname(key)) in LANG_ALTS:
-                    add_array(node, AltList([_clean(val)]))
+    def _setitem_update(self, key, val, qkey):
+        # Locate existing node to replace
+        node, attrib, _oldval, _parent = next(self._get_elements(key))
+        if attrib:
+            if not isinstance(val, str):
+                if qkey == self._qname('dc:creator'):
+                    # dc:creator incorrectly created as an attribute - we're
+                    # replacing it anyway, so remove the old one
+                    del node.attrib[qkey]
+                    self._setitem_add_array(node, _clean(val))
                 else:
-                    node.text = _clean(val)
+                    raise TypeError(f"Setting {key} to {val} with type {type(val)}")
             else:
-                raise TypeError(val)
-        except StopIteration:
-            # Insert a new node
-            rdf = self._get_rdf_root()
+                node.set(attrib, _clean(val))
+        elif isinstance(val, (list, set)):
+            for child in node.findall('*'):
+                node.remove(child)
+            self._setitem_add_array(node, val)
+        elif isinstance(val, str):
+            for child in node.findall('*'):
+                node.remove(child)
             if str(self._qname(key)) in LANG_ALTS:
-                val = AltList([_clean(val)])
-            if isinstance(val, (list, set)):
-                rdfdesc = etree.SubElement(
-                    rdf,
-                    str(QName(XMP_NS_RDF, 'Description')),
-                    attrib={str(QName(XMP_NS_RDF, 'about')): ''},
-                )
-                node = etree.SubElement(rdfdesc, self._qname(key))
-                add_array(node, val)
-            elif isinstance(val, str):
-                _rdfdesc = etree.SubElement(  # lgtm [py/unused-local-variable]
-                    rdf,
-                    str(QName(XMP_NS_RDF, 'Description')),
-                    attrib={
-                        QName(XMP_NS_RDF, 'about'): '',
-                        self._qname(key): _clean(val),
-                    },
-                )
+                self._setitem_add_array(node, AltList([_clean(val)]))
             else:
-                raise TypeError(val) from None
+                node.text = _clean(val)
+        else:
+            raise TypeError(f"Setting {key} to {val} with type {type(val)}")
+
+    def _setitem_insert(self, key, val):
+        rdf = self._get_rdf_root()
+        if str(self._qname(key)) in LANG_ALTS:
+            val = AltList([_clean(val)])
+        if isinstance(val, (list, set)):
+            rdfdesc = etree.SubElement(
+                rdf,
+                str(QName(XMP_NS_RDF, 'Description')),
+                attrib={str(QName(XMP_NS_RDF, 'about')): ''},
+            )
+            node = etree.SubElement(rdfdesc, self._qname(key))
+            self._setitem_add_array(node, val)
+        elif isinstance(val, str):
+            _rdfdesc = etree.SubElement(  # lgtm [py/unused-local-variable]
+                rdf,
+                str(QName(XMP_NS_RDF, 'Description')),
+                attrib={
+                    QName(XMP_NS_RDF, 'about'): '',
+                    self._qname(key): _clean(val),
+                },
+            )
+        else:
+            raise TypeError(f"Setting {key} to {val} with type {type(val)}") from None
 
     @ensure_loaded
     def __setitem__(self, key: Union[str, QName], val: Union[Set[str], List[str], str]):
@@ -787,7 +818,6 @@ class PdfMetadata(MutableMapping):
             raise KeyError(key) from None
 
     @property
-    @ensure_loaded
     def pdfa_status(self) -> str:
         """Returns the PDF/A conformance level claimed by this PDF, or False
 
@@ -796,10 +826,14 @@ class PdfMetadata(MutableMapping):
         conformant.
 
         Returns:
-            str: The conformance level of the PDF/A, or an empty string if the
+            The conformance level of the PDF/A, or an empty string if the
             PDF does not claim PDF/A conformance. Possible valid values
             are: 1A, 1B, 2A, 2B, 2U, 3A, 3B, 3U.
         """
+        # do same as @ensure_loaded - mypy can't handle decorated property
+        if not self._xmp:
+            self._load()
+
         key_part = QName(XMP_NS_PDFA_ID, 'part')
         key_conformance = QName(XMP_NS_PDFA_ID, 'conformance')
         try:
@@ -808,7 +842,6 @@ class PdfMetadata(MutableMapping):
             return ''
 
     @property
-    @ensure_loaded
     def pdfx_status(self) -> str:
         """Returns the PDF/X conformance level claimed by this PDF, or False
 
@@ -817,9 +850,13 @@ class PdfMetadata(MutableMapping):
         conformant.
 
         Returns:
-            str: The conformance level of the PDF/X, or an empty string if the
+            The conformance level of the PDF/X, or an empty string if the
             PDF does not claim PDF/X conformance.
         """
+        # do same as @ensure_loaded - mypy can't handle decorated property
+        if not self._xmp:
+            self._load()
+
         pdfx_version = QName(XMP_NS_PDFX_ID, 'GTS_PDFXVersion')
         try:
             return self[pdfx_version]
