@@ -1,7 +1,7 @@
 import pytest
 
 import pikepdf
-from pikepdf import Pdf
+from pikepdf import Name, Pdf
 
 
 @pytest.fixture
@@ -20,3 +20,11 @@ def test_foreign_linearization(vera):
 @pytest.mark.parametrize('msg, expected', [('QPDF', 'pikepdf.Pdf')])
 def test_translate_qpdf(msg, expected):
     assert pikepdf._qpdf._translate_qpdf(msg) == expected
+
+
+@pytest.mark.parametrize('filter_', ['/ASCII85Decode', '/ASCIIHexDecode'])
+def test_char_out_of_range(filter_):
+    p = pikepdf.new()
+    st = pikepdf.Stream(p, b'\xba\xad', Filter=Name(filter_))
+    with pytest.raises(pikepdf.DataDecodingError):
+        st.read_bytes()
