@@ -45,8 +45,8 @@ def test_none():
 
 
 def test_booleans():
-    assert encode(True) == True
-    assert encode(False) == False
+    assert encode(True) == True  # noqa: E712
+    assert encode(False) == False  # noqa: E712
 
 
 @given(characters(min_codepoint=0x20, max_codepoint=0x7F))
@@ -121,52 +121,6 @@ def test_decimal_from_float(f):
             Object.parse(str(d))
 
 
-@given(lists(integers(-10, 10), min_size=0, max_size=10))
-def test_list(array):
-    a = pikepdf.Array(array)
-    assert a == array
-
-
-@given(lists(lists(integers(1, 10), min_size=1, max_size=5), min_size=1, max_size=5))
-def test_nested_list(array):
-    a = pikepdf.Array(array)
-    assert a == array
-
-
-@given(
-    recursive(
-        integers(1, 10) | booleans(),
-        lambda children: lists(children),  # pylint: disable=unnecessary-lambda
-        max_leaves=20,
-    )
-)
-def test_nested_list2(array):
-    assume(isinstance(array, list))
-    a = pikepdf.Array(array)
-    assert a == array
-
-
-def test_list_apis():
-    a = pikepdf.Array([1, 2, 3])
-    a[1] = None
-    assert a[1] is None
-    assert len(a) == 3
-    del a[1]
-    assert len(a) == 2
-    a[-1] = Name('/Foo')
-    with pytest.raises(IndexError):
-        a[-5555] = Name.Foo
-    assert a == pikepdf.Array([1, Name.Foo])
-    a.append(4)
-    assert a == pikepdf.Array([1, Name.Foo, 4])
-    a.extend([42, 666])
-    assert a == pikepdf.Array([1, Name.Foo, 4, 42, 666])
-    with pytest.raises(ValueError, match='object is not a dictionary'):
-        del a.ImaginaryKey
-    with pytest.raises(TypeError, match=r"items\(\) not available"):
-        a.items()
-
-
 @skip_if_pypy
 def test_stack_depth():
     a = [42]
@@ -178,7 +132,7 @@ def test_stack_depth():
         with pytest.raises(RecursionError):
             assert encode(a) == a
         with pytest.raises(RecursionError):
-            encode(a) == encode(a)  # pylint: disable=expression-not-assigned
+            assert encode(a) == encode(a)  # pylint: disable=expression-not-assigned
         with pytest.raises(RecursionError):
             repr(a)
     finally:
@@ -199,15 +153,93 @@ def test_bytes():
         Name(b'/bytes')
 
 
-def test_len_array():
-    assert len(Array([])) == 0
-    assert len(Array()) == 0
-    assert len(Array([3])) == 1
+class TestArray:
+    def test_len_array(self):
+        assert len(Array([])) == 0
+        assert len(Array()) == 0
+        assert len(Array([3])) == 1
 
+    def test_wrap_array(self):
+        assert Name('/Foo').wrap_in_array() == Array([Name('/Foo')])
+        assert Array([42]).wrap_in_array() == Array([42])
 
-def test_wrap_array():
-    assert Name('/Foo').wrap_in_array() == Array([Name('/Foo')])
-    assert Array([42]).wrap_in_array() == Array([42])
+    @given(lists(integers(-10, 10), min_size=0, max_size=10))
+    def test_list(self, array):
+        a = pikepdf.Array(array)
+        assert a == array
+
+    @given(
+        lists(lists(integers(1, 10), min_size=1, max_size=5), min_size=1, max_size=5)
+    )
+    def test_nested_list(self, array):
+        a = pikepdf.Array(array)
+        assert a == array
+
+    @given(
+        recursive(
+            integers(1, 10) | booleans(),
+            lambda children: lists(children),  # pylint: disable=unnecessary-lambda
+            max_leaves=20,
+        )
+    )
+    def test_nested_list2(self, array):
+        assume(isinstance(array, list))
+        a = pikepdf.Array(array)
+        assert a == array
+
+    def test_array_of_array(self):
+        a = Array([1, 2])
+        a2 = Array(a)
+        assert a == a2
+        assert a is not a2
+
+    def test_array_of_primitives_eq(self):
+        a = Array([True, False, 0, 1, 42, 42.42])
+        b = Array([True, False, 0, 1, 42, 42.42])
+        assert a == b
+        c = Array([1.0, 0.0, 0.0, 1.0, 42.0, 42.42])
+        assert a == c
+
+    def test_list_apis(self):
+        a = pikepdf.Array([1, 2, 3])
+        a[1] = None
+        assert a[1] is None
+        assert len(a) == 3
+        del a[1]
+        assert len(a) == 2
+        a[-1] = Name('/Foo')
+        with pytest.raises(IndexError):
+            a[-5555] = Name.Foo
+        assert a == pikepdf.Array([1, Name.Foo])
+        a.append(4)
+        assert a == pikepdf.Array([1, Name.Foo, 4])
+        a.extend([42, 666])
+        assert a == pikepdf.Array([1, Name.Foo, 4, 42, 666])
+        with pytest.raises(ValueError, match='object is not a dictionary'):
+            del a.ImaginaryKey
+        with pytest.raises(TypeError, match=r"items\(\) not available"):
+            a.items()
+
+    def test_array_contains(self):
+        a = pikepdf.Array([Name.One, Name.Two])
+        assert Name.One in a
+        assert Name.Two in a
+        assert Name.N not in a
+
+        a = pikepdf.Array([1, 2, 3])
+        assert 1 in a
+        assert 3 in a
+        assert 42 not in a
+
+        with pytest.raises(TypeError):
+            assert 'forty two' not in a
+        with pytest.raises(TypeError):
+            assert b'forty two' not in a
+        assert pikepdf.String('forty two') not in a
+
+        a = pikepdf.Array(['1234', b'\x80\x81\x82'])
+        assert pikepdf.String('1234') in a
+        assert pikepdf.String(b'\x80\x81\x82') in a
 
 
 def test_no_len():
@@ -228,7 +260,7 @@ class TestName:
 
     def test_unslashed_name(self):
         with pytest.raises(ValueError, match='must begin with'):
-            Name('Monty') not in []  # pylint: disable=expression-not-assigned
+            assert Name('Monty') not in []  # pylint: disable=expression-not-assigned
 
     def test_empty_name(self):
         with pytest.raises(ValueError):
@@ -250,6 +282,10 @@ class TestName:
     def test_bytes_of_name(self):
         assert bytes(Name.ABC) == b'/ABC'
 
+    def test_name_from_name(self):
+        foo = Name('/Foo')
+        assert Name(foo) == foo
+
 
 class TestHashViolation:
     def check(self, a, b):
@@ -301,10 +337,7 @@ class TestRepr:
                 '/None': None,
             }
         )
-        if LooseVersion(pikepdf.__libqpdf_version__) >= LooseVersion('10.2.0'):
-            short_pi = '3.14'
-        else:
-            short_pi = '3.140000'
+        short_pi = '3.14'
         expected = (
             """\
             pikepdf.Dictionary({
@@ -353,12 +386,18 @@ class TestRepr:
             pdf.Root.Circular.Parent = pdf.make_indirect(pdf.Root.Circular)
             assert '.get_object' in repr(pdf.Root.Circular)
 
+    def test_repr_indirect_page(self, resources):
+        with pikepdf.open(resources / 'outlines.pdf') as outlines:
+            assert 'from_objgen' in repr(outlines.Root.Pages.Kids)
+            # An indirect page reference in the Dests name tree
+            assert 'from_objgen' in repr(outlines.Root.Names.Dests.Kids[0].Names[1])
+
 
 def test_operator_inline(resources):
     with pikepdf.open(resources / 'image-mono-inline.pdf') as pdf:
         instructions = parse_content_stream(pdf.pages[0], operators='BI ID EI')
         assert len(instructions) == 1
-        operands, operator = instructions[0]
+        _operands, operator = instructions[0]
         assert operator == pikepdf.Operator("INLINE IMAGE")
 
 
@@ -415,11 +454,6 @@ class TestDictionary:
         assert d.get(Name.A) == 'a'
         assert d.get(Name.Resources, 42) == 42
 
-    def test_nonpage(self):
-        d = pikepdf.Dictionary(A='a')
-        with pytest.raises(TypeError):
-            d.images  # pylint: disable=pointless-statement
-
     def test_bad_name_init(self):
         with pytest.raises(KeyError, match=r"must begin with '/'"):
             pikepdf.Dictionary({'/Slash': 'dot', 'unslash': 'error'})
@@ -447,7 +481,17 @@ class TestDictionary:
     def test_wrong_contains_type(self):
         d = pikepdf.Dictionary()
         with pytest.raises(TypeError, match="can only contain Names"):
-            pikepdf.Array([3]) in d
+            assert pikepdf.Array([3]) not in d
+
+    def test_dict_bad_params(self):
+        with pytest.raises(ValueError):
+            Dictionary({'/Foo': 1}, Bar=2)
+
+    def test_dict_of_dict(self):
+        d = Dictionary(One=1, Two=2)
+        d2 = Dictionary(d)
+        assert d == d2
+        assert d is not d2
 
 
 def test_not_convertible():
@@ -648,7 +692,10 @@ def test_stream_as_dict(abcxyz_stream):
     stream[Name.TestKeyNameAccess] = True
     assert len(stream.keys()) == 4  # Streams always have a /Length
 
-    assert all((v == len(stream.read_bytes()) or v == True) for k, v in stream.items())
+    assert all(
+        (v == len(stream.read_bytes()) or v == True)  # noqa: E712
+        for k, v in stream.items()
+    )
 
     assert stream.stream_dict.TestAttrAccess
 
@@ -689,11 +736,6 @@ def test_stream_dict_oneshot():
     assert stream3.One == 1
 
 
-def test_dict_bad_params():
-    with pytest.raises(ValueError):
-        Dictionary({'/Foo': 1}, Bar=2)
-
-
 def test_stream_bad_params():
     p = pikepdf.new()
     with pytest.raises(TypeError, match='data'):
@@ -708,20 +750,6 @@ def test_stream_no_dangling_stream_on_failure():
     assert len(p.objects) == num_objects, "A dangling object was created"
 
 
-def test_dict_of_dict():
-    d = Dictionary(One=1, Two=2)
-    d2 = Dictionary(d)
-    assert d == d2
-    assert d is not d2
-
-
-def test_array_of_array():
-    a = Array([1, 2])
-    a2 = Array(a)
-    assert a == a2
-    assert a is not a2
-
-
 def test_object_mapping(sandwich):
     object_mapping = sandwich.pages[0].images
     assert '42' not in object_mapping
@@ -729,14 +757,13 @@ def test_object_mapping(sandwich):
     assert '/R12' in object_mapping.keys()
 
 
-@needs_libqpdf_v('10.3.0', reason="behavior of replace improved in v10.3.0")
 def test_replace_object(sandwich):
     d = Dictionary(Type=Name.Dummy)
-    sandwich._replace_object(sandwich.pages[0].objgen, d)
-    assert sandwich.pages[0] == d
+    profile = sandwich.Root.OutputIntents[0].DestOutputProfile.objgen
+    sandwich._replace_object(profile, d)
+    assert sandwich.Root.OutputIntents[0].DestOutputProfile == d
 
 
-@needs_libqpdf_v('10.3.0', reason="behavior of swap improved in v10.3.0")
 def test_swap_object(resources):
     with Pdf.open(resources / 'fourpages.pdf') as pdf:
         pdf.pages[0].MarkPage0 = True
